using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using Outlook = Microsoft.Office.Interop.Outlook;

namespace MyOutlookAddin1
{
	/// <summary>
	///   Add-in Express Add-in Module
	/// </summary>
	[GuidAttribute("31F62B72-CD8C-409F-9D5A-CD13F3D9D63C"), ProgId("MyOutlookAddinCS.AddinModule")]
	public partial class AddinModule : AddinExpress.MSO.ADXAddinModule, Logging.ILogPaneManager 
	{
        public AddinModule()
        {
            System.Windows.Forms.Application.EnableVisualStyles();
            InitializeComponent();

            Logging.Log.Instance.NewData += Log_NewData;
            // Please add any initialization code to the AddinInitialize event handler
        }

        public const bool showPanesAtStartup = true;
        const bool copyRibbonXmlToClipboard = false;

        private OutlookItemsEventsClass1 itemsEvents = null;

        #region Add-in Express automatic code

        // Required by Add-in Express - do not modify
        // the methods within this region

        public override System.ComponentModel.IContainer GetContainer()
		{
			if (components == null)
				components = new System.ComponentModel.Container();
			return components;
		}

		[ComRegisterFunctionAttribute]
		public static void AddinRegister(Type t)
		{
			AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
		}

		[ComUnregisterFunctionAttribute]
		public static void AddinUnregister(Type t)
		{
			AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
		}

		public override void UninstallControls()
		{
			base.UninstallControls();
		}

		#endregion

		#region Useful properties
		public static new AddinModule CurrentInstance
		{
			get
			{
				return AddinExpress.MSO.ADXAddinModule.CurrentInstance as AddinModule;
			}
		}		
		public Outlook._Application OutlookApp
		{
			get
			{
				return (HostApplication as Outlook._Application);
			}
		}
        #endregion

        #region Add-in events
        private void AddinModule_AddinStartupComplete(object sender, EventArgs e)
		{
            this.adxRibbonButtonPaneVisibilityInfo.Visible = showPanesAtStartup;
            itemsEvents = new OutlookItemsEventsClass1(this);
			itemsEvents.ConnectTo(AddinExpress.MSO.ADXOlDefaultFolders.olFolderInbox, true);
            Logging.Log.Add("AddinModule.AddinModule_AddinStartupComplete", "");
		}
        bool isShutdown = false;
		private void AddinModule_AddinBeginShutdown(object sender, EventArgs e)
		{
            isShutdown = true;
            if (itemsEvents != null)
			{
				itemsEvents.RemoveConnection();
				itemsEvents = null;
			}
		}
        #endregion

        #region Ribbon events of the module
        private void AddinModule_OnRibbonBeforeCreate(object sender, string ribbonId)
        {
            this.adxRibbonButtonPaneVisibilityInfo.Visible = showPanesAtStartup;

            this.adxRibbonContextMenu2.ContextMenuNames.AddRange(new string[] {
                "Outlook.Mail.Compose.ContextMenuText",
                "Outlook.Explorer.ContextMenuText",
                "Outlook.Mail.Compose.ContextMenuTextTable",
                "Outlook.Explorer.ContextMenuTextTable",
                "Outlook.Mail.Read.ContextMenuReadOnlyMailText",
                "Outlook.Explorer.ContextMenuReadOnlyMailText",
                "Outlook.Mail.Read.ContextMenuReadOnlyMailTextTable",
                "Outlook.Explorer.ContextMenuReadOnlyMailTextTable"});
        }
        private void AddinModule_OnRibbonBeforeLoad(object sender, AddinExpress.MSO.ADXRibbonBeforeLoadEventArgs e)
        {
            if (copyRibbonXmlToClipboard)
            {
                System.Windows.Forms.Clipboard.SetText(e.Xml);
            }
        }
        #endregion 

        #region Host application events
        private void adxOutlookEvents_ExplorerSelectionChange(object sender, object explorer)
        {
            string contextInfo = new ContextInfo(explorer).ToString();
            Logging.Log.Add("AddinModule.adxOutlookEvents_ExplorerSelectionChange", contextInfo);
        }
        private void adxOutlookEvents_ExplorerBeforeFolderSwitch(object sender, AddinExpress.MSO.ADXOlExplorerBeforeFolderSwitchEventArgs e)
        {
            string contextInfo = new ContextInfo(e.Explorer).ToString();
            Logging.Log.Add("AddinModule.adxOutlookEvents_ExplorerBeforeFolderSwitch", contextInfo);
        }
        private void adxKeyboardShortcut1_Action(object sender)
        {
            object window = null;
            try
            {
                string message = string.Format("You've pressed '{0}'", (sender as AddinExpress.MSO.ADXKeyboardShortcut).ShortcutText);
                System.Windows.Forms.MessageBox.Show(message, AddinName);

                window = OutlookApp.ActiveWindow();
                if (window != null)
                {
                    string contextInfo = new ContextInfo(window).ToString();
                    Logging.Log.Add("AddinModule.adxKeyboardShortcut1_Action", contextInfo);
                }
            }
            catch (Exception ex)
            {
                Logging.Log.Add("AddinModule.adxKeyboardShortcut1_Action", ex);
            }
            finally
            {
                if (window != null) Marshal.ReleaseComObject(window);
            }
        }
        #endregion

        #region Events of Command bar controls
		private void CommandBarButton_CommonClickHandler(object sender)
		{
			string buttonCaption = (sender as AddinExpress.MSO.ADXCommandBarButton).Caption;
			ContextInfo contextInfo = new ContextInfo(null);			
			string message = string.Format("Button '{0}' clicked.\n\nContext:\n{1}\n\nHost:\n{2}", buttonCaption, contextInfo, ContextInfo.GetHostSummary());
            System.Windows.Forms.MessageBox.Show(message, "CommandBarButton_CommonClickHandler");
        }
        #endregion

        #region Events of Ribbon controls
        private void RibbonButton_CommonClickHandler(object sender, AddinExpress.MSO.IRibbonControl control, bool pressed)
        {
            string buttonCaption = (sender as AddinExpress.MSO.ADXRibbonButton).Caption;
            object context = control.Context;
            ContextInfo contextInfo = new ContextInfo(context);
            if (context != null) Marshal.ReleaseComObject(context); context = null;
            string message = string.Format("Button '{0}' clicked.\n\nContext:\n{1}\n\nHost:\n{2}", buttonCaption, contextInfo, ContextInfo.GetHostSummary());
            System.Windows.Forms.MessageBox.Show(message, AddinName);
            Logging.Log.Add("AddinModule.RibbonButton_CommonClickHandler", message);
        }
        private void adxBackstageRegularButton1_OnClick(object sender, AddinExpress.MSO.IRibbonControl control)
        {
            System.Windows.Forms.MessageBox.Show("You've clicked the " + control.Id + " button.", AddinName);
        }
        #endregion

        #region Dynamic Ribbon Menu
        Dictionary<int, string> db = new Dictionary<int, string>(); // imitates source of changing data 
        int counter = 0;

        private void adxRibbonButtonAddRecord_OnClick(object sender, AddinExpress.MSO.IRibbonControl control, bool pressed)
        {
            int id = counter++;

            object context = control.Context;
            ContextInfo contextInfo = new ContextInfo(context);
            string windowCaption = contextInfo.WindowCaption;
            if (context != null) Marshal.ReleaseComObject(context); context = null;
            string time = System.DateTime.Now.ToLongTimeString();
            string value = string.Format("{0} Window={1}", time, windowCaption);
            if (db.Count == 0) db.Add(id, "separator");
            id = counter++;
            db.Add(id, value);

            this.adxRibbonMenuDynamic.Caption = string.Format("Menu ({0} items)", db.Count - 1);
        }
        private void adxRibbonButtonClearRecords_OnClick(object sender, AddinExpress.MSO.IRibbonControl control, bool pressed)
        {
            db.Clear();
            this.adxRibbonMenuDynamic.Caption = string.Format("Menu ({0} items)", 0);
        }
        private void adxRibbonMenuDynamic_OnCreate(object sender, AddinExpress.MSO.ADXRibbonCreateMenuEventArgs e)
        {
            e.Clear(); //remove previously created controls
            foreach (KeyValuePair<int, string> pair in db)
            {
                if (pair.Value.ToLower() == "separator") 
                {
                    AddinExpress.MSO.ADXRibbonMenuSeparator separator = 
                        new AddinExpress.MSO.ADXRibbonMenuSeparator(this.components);
                    separator.Id = "newControl" + pair.Key.ToString();
                    separator.Caption = "this is a separator";
                    separator.Ribbons = this.adxRibbonMenuDynamic.Ribbons;
                    this.adxRibbonMenuDynamic.Controls.Add(separator);
                    e.AddControl(separator);
                }
                else
                {
                    AddinExpress.MSO.ADXRibbonButton newButton = 
                        new AddinExpress.MSO.ADXRibbonButton(this.components); // passing this.components means ADX will know about this button
                    newButton.Id = "newControl" + pair.Key.ToString(); // id must start with a letter
                    newButton.Caption = pair.Value;
                    newButton.Ribbons = this.adxRibbonMenuDynamic.Ribbons;
                    newButton.OnClick += RibbonButton_CommonClickHandler;
                    e.AddControl(newButton);
                }
            }
        }
        #endregion Dynamic Ribbon Menu

        #region Pane-related
        private void adxRibbonButtonPaneVisibilityInfo_OnClick(object sender, AddinExpress.MSO.IRibbonControl control, bool pressed)
        {
            System.Windows.Forms.MessageBox.Show(paneDefaultVisibilityMessage, AddinName);
            adxRibbonButtonPaneVisibilityInfo.Visible = false;
            Logging.Log.Add("AddinModule.adxRibbonButtonPaneVisibilityInfo_OnClick", "paneDefaultVisibilityMessage shown");
        }

        bool showButtonInfoShown = false;
        private void adxRibbonButtonShowPane_OnClick(object sender, AddinExpress.MSO.IRibbonControl control, bool pressed)
        {            
            object context = control.Context;
            Logging.ILogPane pane = GetPane(adxOlFormsCollectionItem1, context); 
            if (pane != null)
            {
                ShowInfoMessage(context);
                pane.Show();
            }
            if (context != null) Marshal.ReleaseComObject(context);
        }
        private void ShowInfoMessage(object context)
        {
            if (!showButtonInfoShown)
            {
                if (context is Outlook.Explorer)
                {
                    showButtonInfoShown = true;
                    System.Windows.Forms.MessageBox.Show(paneChangeVisibilityMessage, AddinName);
                    Logging.Log.Add("AddinModule.ShowInfoMessage", "paneChangeVisibilityMessage shown");
                }
            }
        }
        private void adxRibbonButtonHidePane_OnClick(object sender, AddinExpress.MSO.IRibbonControl control, bool pressed)
        {
            object context = control.Context;
            Logging.ILogPane pane = GetPane(adxOlFormsCollectionItem1, context);
            if (pane != null)
            {
                ShowInfoMessage(context);
                pane.Hide();
            }
            if (context != null) Marshal.ReleaseComObject(context);
        }
        private void adxRibbonButtonShowPane_PropertyChanging(object sender, AddinExpress.MSO.ADXRibbonPropertyChangingEventArgs e)
        {
            if (e.PropertyType == AddinExpress.MSO.ADXRibbonControlPropertyType.Visible)
            {
                Logging.Log.Add("AddinModule.adxRibbonButtonShowPane_PropertyChanging", "ADXRibbonControlPropertyType.Visible");
                bool result = false;
                object context = e.Context;
                Logging.ILogPane pane = GetPane(adxOlFormsCollectionItem1, context);
                if (pane != null)
                {
                    result = pane.IsPaneShown;
                }
                e.Value = !result;
            }
        }
        private void adxRibbonButtonHidePane_PropertyChanging(object sender, AddinExpress.MSO.ADXRibbonPropertyChangingEventArgs e)
        {
            if (e.PropertyType == AddinExpress.MSO.ADXRibbonControlPropertyType.Visible)
            {
                Logging.Log.Add("AddinModule.adxRibbonButtonHidePane_PropertyChanging", "ADXRibbonControlPropertyType.Visible");
                bool result = false;
                object context = e.Context;
                Logging.ILogPane pane = GetPane(adxOlFormsCollectionItem1, context);
                if (pane != null)
                {
                    result = pane.IsPaneShown;
                }
                e.Value = result;
            }
        }
        private Logging.ILogPane GetPane(AddinExpress.OL.ADXOlFormsCollectionItem item, object contextWindow)
        {
            if (contextWindow == null)
            {
                Logging.Log.Add("AddinModule.GetPane", "context not provided");
                return null;
            }
            Logging.ILogPane pane =
                item.GetForm(ExplorerOrInspectorObj: contextWindow) as Logging.ILogPane;
            return pane;
        }

        private void adxOlFormsManager1_ADXBeforeFormInstanceCreate(object sender, AddinExpress.OL.BeforeFormInstanceCreateEventArgs args)
        {
            // you can find the context using arg.ExplorerObj or args.InspectorObj
            Logging.Log.Add("AddinModule.adxOlFormsManager1_ADXBeforeFormInstanceCreate", "let a new pane instance create");
        }
        #endregion Pane-related

        #region Log-related
        private void Log_NewData(object sender, Logging.LogRecord e)
        {
            System.Diagnostics.Debug.WriteLine(this.AddinName + ": " + e.ToString());
            if (NewLogData != null) NewLogData(this as Logging.ILogPaneManager, e);
        }

        public event EventHandler<Logging.LogRecord> NewLogData;
        List<Logging.LogRecord> Logging.ILogPaneManager.GetRecords()
        {
            return Logging.Log.Instance.GetData();
        }
        void Logging.ILogPaneManager.PaneVisibilityChanged(Logging.ILogPane logPane)
        {
            if (isShutdown) return;
            this.adxRibbonButtonShowPane.Invalidate();
            this.adxRibbonButtonHidePane.Invalidate();
        }
        #endregion Log-related

        const string paneChangeVisibilityMessage = "This will hide or show the pane instance associated with this Explorer window. Note that depending on the adxOlFormsCollectionItem1.Cached property this instance may be used for all folders or for this folder only (default). That is, by default, switching to another folder wiil show a different pane instance. Each instance manages a boolean flag indicating whether the pane will show or not.";
        const string paneDefaultVisibilityMessage = "To change the default visibility of the Add-in Express panes that the add-in creates, set showPanesAtStartup=false in the code of the add-in module and compile the project";
    }
}
